/*
 ********************************************************************************
 *
 *  M O U S E B T N . H
 *
 *  Purpose:
 *      Demonstrate how a VxD driver can call into WDM in order to preserve 
 *      existing VxD API interface
 *
 *  Copyright 1997  Microsoft Corp.
 *
 *  Author:     Dr. Iver
 *
 *
 ********************************************************************************
 */

#define WANTVXDWRAPS
#include <basetyps.h>
#include <wdm.h>
#include <BASEDEF.H>
#include <vmm.h>
#include <vwin32.h>
#include <debug.h>
#include <winerror.h>

#ifdef  DEBUG
    extern UINT dbgOpt;

    #define DBGOUT(msg_in_parens) if (dbgOpt) { _Debug_Printf_Service("\r\nMOUSEBTN> "); _Debug_Printf_Service msg_in_parens; _Debug_Printf_Service("\r\n"); }
    #define DBGERR(msg_in_parens) { _Debug_Printf_Service("\r\n *** MOUSEBTN ERROR *** \r\nMOUSEBTN> "); _Debug_Printf_Service msg_in_parens; _Debug_Printf_Service("\r\n"); }
    #define DBGBREAK() { _asm { int 3 } }

    #undef ASSERT
    #define ASSERT(x) if (!(x)){ DBGERR(("Assertion failed:" #x)); DBGBREAK(); }

#else
    #define DBGOUT(msg_in_parens)
    #define DBGERR(msg_in_parens)
    #define DBGBREAK()
#endif

//
// Use the same device ID that you used for your original vxd.
// 
#define MAPSYS_DEVICE_ID 0x9624

/*XLATOFF*/
#define MAPSYS_Service Declare_Service
#pragma warning (disable:4003) // turn off not enough params warning
/*XLATON*/

// 
// 'C' Language represnetation of mapsys.sys VXD service table.
//  Functions must be in the same order as in mapsysa.inc.
//

Begin_Service_Table(MAPSYS)

MAPSYS_Service(MAPSYS_GetVersion, LOCAL)
MAPSYS_Service(MAPSYS_OpenMouse, LOCAL)
MAPSYS_Service(MAPSYS_ReadButtons, LOCAL)
MAPSYS_Service(MAPSYS_CloseMouse, LOCAL)

End_Service_Table(MAPSYS)

DWORD VXDINLINE MAPSYS_GetVersion(void)
{
    DWORD dw;

    VxDCall(MAPSYS_GetVersion)
    _asm mov dw, eax            // return valued passed through eax
    return dw;
}

HANDLE VXDINLINE MAPSYS_OpenMouse()
{
    HANDLE h;

    VxDCall(MAPSYS_OpenMouse)
    _asm mov h, eax
    return h;
}

NTSTATUS VXDINLINE MAPSYS_ReadButtons(HANDLE h, UINT *pButtonStatus)
{
    NTSTATUS ntStatus;

    _asm mov eax, h             // handle parameter passed through eax
    _asm mov ecx, pButtonStatus // OUT button status pointer passed through ecx

    VxDCall(MAPSYS_ReadButtons)

    _asm mov ntStatus, eax      // return value passed through eax
    return ntStatus;
}

DWORD VXDINLINE MAPSYS_CloseMouse(HANDLE h)
{
    DWORD dw;

    _asm mov eax, h               // handle passed through eax 
    VxDCall(MAPSYS_CloseMouse)
    _asm mov dw, eax              // return value passed through eax        
    return dw;              
}
