/*
 ********************************************************************************
 *
 *  M O U S E B T N . C
 *
 *	Purpose:
 *  	Demonstrate how a VxD driver can call into SYS driver in order to preserve 
 *		existing VxD API interface
 *
 *		Mousebtn.c is the main file for the "legacy" app.  The goal of the
 *		sample is to change the mousebtn.vxd driver to work with a USB 
 *		mouse such that this file doesn't have to be modified (i.e. 
 *		preserve the VxD interface).
 *
 *  Copyright 1997  Microsoft Corp.
 *
 *  Author:     Dr. Iver
 *
 *
 ********************************************************************************
 */

#include <windows.h>
#include <winbase.h>
#include <stdio.h>
#include "../mousemon.h"

/*****************************************************************
 *
 * ScanKey
 * 
 * Purpose:
 *		Look for keypress, return character if available
 *
 * Parameters:
 *	    HANDLE hStdIn  // from GetStdHandle(STD_INPUT_HANDLE);
 *
 * Return:
 *		Character if available, '\0' if no characters available  
 *
 *****************************************************************/ 

char ScanKey( HANDLE hStdIn )
{
	INPUT_RECORD IRbuf;
	ULONG ul;

	if ( PeekConsoleInput(  // Is there any data available?
			hStdIn,		// handle to a console input buffer 
			&IRbuf,		// pointer to the buffer for peek data 
			1,			// number of records to read 
			&ul ) 		// pointer to number of records read 
			&& ( 1 == ul ) )
	{	
		if ( ReadConsoleInput(  // Get char from keyboard
				hStdIn,		
				&IRbuf,		
				1,			
				&ul ) )		
		{
			if ( (1 == ul) && 		// yes there is 1 record of data
				 (KEY_EVENT == IRbuf.EventType) && // data is from keyboard
				 (IRbuf.Event.KeyEvent.bKeyDown) ) // key is being pressed down
			{
				return ( IRbuf.Event.KeyEvent.uChar.AsciiChar );
			}
		}
	}
	return ('\0'); 		// no data available, return NULL char
}
															 
void main()
{
	int iInVal;
	int iOutVal;
	int nBytes = 0;
	HANDLE hDevice;
	HANDLE hStdIn = GetStdHandle(STD_INPUT_HANDLE);
	char ch;
	OVERLAPPED ov;

	if ( (hDevice=CreateFile(	"\\\\.\\mousebtn.vxd", 0, FILE_SHARE_READ, NULL, 0, 
								FILE_FLAG_DELETE_ON_CLOSE, NULL ) )
		         ==  INVALID_HANDLE_VALUE )
	{
		printf( "Error %X opening mapvxd.vxd\n", GetLastError() );
		exit(1);
	}

	while( 1 )
	{
		Sleep ( 50 );  // simulate app processing data 
		if ( !DeviceIoControl(	hDevice,
							DIOC_GETMOUSEBUTTON,
							&iInVal,
							sizeof( int ),
							&iOutVal,
							sizeof( int ),
							&nBytes,
							&ov ) )
		{
			printf( "Error %X calling DeviceIOControl\n", GetLastError() );
		}
		ch = ScanKey( hStdIn );
		if ( ('q' == ch) || ('Q' == ch ) )
		{	
			break;		// quit when user presses 'Q' on keyboard
		}
		if ( iOutVal & 1)
			printf( "Left mouse button pressed\n" );
		else if ( iOutVal & 2)
			printf( "Right mouse button pressed\n" );
	}
	CloseHandle( hDevice );
	CloseHandle( hStdIn );
}
