/*
 ********************************************************************************
 *
 *  M A P S Y S . C
 *
 *	Purpose:
 *  	Demonstrate how a VxD driver can call into SYS driver in order to preserve 
 *		existing VxD API interface
 *
 *		This is the main file for the MaySys.Sys driver that passes calls
 *		from the modified mousebtn.vxd to the new "WDM" driver (wdmdrv.sys).
 *
 *  Copyright 1997  Microsoft Corp.
 *
 *  Author:     Dr. Iver
 *
 *
 ********************************************************************************
 */

#define WANTVXDWRAPS
#define INITGUID

#include <wdm.h>
#include <basedef.h>
#include <vmm.h>
#include <winerror.h>
#include <vmmreg.h>
#include <vxdwraps.h>
#include <configmg.h>
#include <regstr.h>
#include <debug.h>
#include <vwin32.h>
#include "..\wdmdrv\wdmdrv.h"

#ifdef DBG	// WDM DDK build enviornment uses DBG
#define DEBUG
#endif 

#ifdef  DEBUG
    extern UINT dbgOpt;

    #define DBGOUT(msg_in_parens) if (dbgOpt) { DbgPrint("\r\nMAPSYS> "); DbgPrint msg_in_parens; DbgPrint("\r\n"); }
    #define DBGERR(msg_in_parens) { DbgPrint("\r\n *** MAPSYS ERROR *** \r\nMAPSYS> "); DbgPrint msg_in_parens; DbgPrint("\r\n"); }
    #define DBGBREAK() { _asm { int 3 } }

    #undef ASSERT
    #define ASSERT(x) if (!(x)){ DBGERR(("Assertion failed:" #x)); DBGBREAK(); }

#else
    #define DBGOUT(msg_in_parens)
    #define DBGERR(msg_in_parens)
    #define DBGBREAK()
#endif

#ifdef DEBUG
UINT dbgOpt=1;		// display full traces
#endif

VOID MAPSYS_Init_VxD();

NTSTATUS
DriverEntry(
    IN PDRIVER_OBJECT DriverObject,
    IN PUNICODE_STRING RegistryPath
    )
{
	MAPSYS_Init_VxD();			// This routine enables VxD interface
	return STATUS_SUCCESS;
}

//
// Map MAPSYS_OpenMouse Vxd call to OpenMouse function in wdmdrv.sys
//
HANDLE _stdcall _OpenMouseFromVxD()
{
	DBGOUT(( "_OpenMouseFromVxd" ));
	return( OpenMouse() );		// Call OpenMouse in wdmdrv.sys
}

//
// Map MAPSYS_ReadButtons Vxd call to ReadButtons function in wdmdrv.sys
//
NTSTATUS _stdcall _ReadButtonsFromVxD( HANDLE h, UINT *pButtonStatus )
{
	DBGOUT(( "_ReadButtonsFromVxD" ));
	return( ReadButtons( h, pButtonStatus ) );	// Call ReadButtons in wdmdrv.sys
}

//
// Map MAPSYS_CloseMouse Vxd call to CloseMouse function in wdmdrv.sys
//
int _stdcall _CloseMouseFromVxD( HANDLE h )
{
	DBGOUT(( "CloseMouseFromVxD" ));
	return( CloseMouse(h) );
}

